@@ -127,7 +127,8 @@ class QAMServer:
                username='guest',
                password='guest',
                vhost='/',
-               server_id='testqamserver'):
+               server_id='testqamserver',
+               serializer='pickle'):
 
     """
     Initially the amqp-connection, a consumer and a publisher-thread are being created.
@@ -141,6 +142,7 @@ class QAMServer:
     self.kill = False
     self.finished = False
     self.closed = False
+    self.serializer = serializer
 
     self.amqp_credentials = AMQPCredentials(hostname=hostname,
                                             port=port,
@@ -161,7 +163,8 @@ class QAMServer:
 
     self.publisher_thread = PublisherThread(amqp_credentials=self.amqp_credentials,
                                             result_queue = self.result_queue,
-                                            server_id= self.server_id)
+                                            server_id= self.server_id,
+                                            serializer = self.serializer)
     self.publisher_thread.setDaemon(True)
     self.publisher_thread.start()
 
@@ -227,7 +230,8 @@ class QAMServer:
         execute_method_thread = ExecuteMethodThread(self.result_queue,
                                                   message_data,
                                                   self.method_dict,
-                                                  self.instance_dict)
+                                                  self.instance_dict,
+                                                  self.serializer)
         execute_method_thread.start()
 
     self.finished = True
@@ -259,12 +263,13 @@ class ExecuteMethodThread(Thread):
 
   """
 
-  def __init__ (self, result_queue, message_data, method_dict, instance_dict):
+  def __init__ (self, result_queue, message_data, method_dict, instance_dict, serializer):
     Thread.__init__(self)
     self.result_queue = result_queue
     self.message_data = message_data
     self.method_dict = method_dict
     self.instance_dict = instance_dict
+    self.serializer = serializer
    
   def run(self):
     """
@@ -314,9 +319,16 @@ class ExecuteMethodThread(Thread):
     except:
       logger.debug('Exception happend')
       t,v =  sys.exc_info()[:2]
-      logger.debug('catched exception: %s' % repr(v.args))
+      
       sys.exc_clear()
-      result_message = {'id':id, 'result':v.args, 'exception':True, 'internal_exception':-1}
+
+      if self.serializer == 'pickle':
+          logger.debug('pickle: catched exception: %s' % repr(v))
+          result_message = {'id':id, 'result':v, 'exception':True, 'internal_exception':-1}
+      else:
+          logger.debug('json: catched exception: %s' % repr(v.args))
+          result_message = {'id':id, 'result':v.args, 'exception':True, 'internal_exception':-1}
+
     # no exception
     else:
       result_message = {'id':id, 'result':result, 'exception':False, 'internal_exception':-1}
@@ -347,7 +359,7 @@ class PublisherThread(Thread):
     The kill signal tells the PublisherThread to stop sending messages.
   """
 
-  def __init__ (self, amqp_credentials, result_queue, server_id):
+  def __init__ (self, amqp_credentials, result_queue, server_id, serializer):
     Thread.__init__(self)
     self.amqp_credentials = amqp_credentials
     self.result_queue = result_queue
@@ -358,6 +370,7 @@ class PublisherThread(Thread):
                                    password=self.amqp_credentials.password,
                                    host=self.amqp_credentials.vhost)
     self.kill = False
+    self.serializer = serializer
 
   def close(self):
     """
@@ -378,9 +391,11 @@ class PublisherThread(Thread):
       logger.debug('popped message from queue: %s' % repr(result_record['result_message']))
       routing_key = result_record['routing_key']
       logger.debug('create Publisher')
+      logger.debug('Serialization: %s' % self.serializer)
       publisher = Publisher(connection=self.amqpconn,
                             exchange=self.server_id + "_client_exchange",
-                            routing_key=routing_key)
+                            routing_key=routing_key,
+                            serializer=self.serializer)
       logger.debug('created Publisher')
       publisher.send(result_record['result_message'])
       logger.debug('sent message: %s' % repr(result_record['result_message']))
@@ -393,3 +408,5 @@ class MethodNotFoundException(Exception):
   Internal QAMServer Exception, which is thrown, if a non-registered method is being called.
   """
   pass
+
+
